#include <stdio.h>

#include <conio.h>



#define TS5300	0x50

#define TS5400  0x40

#define TS5500	0x60

#define FALSE	0

#define TRUE 	!FALSE



int my_rint (float x) {

	/* math.h roundf and it's relatives are C99 standard functions.

	 * TurboC does not implement it

	 */

	float temp; 

	

	temp = x ; 

	temp = temp + 0.5; 



	return (int)temp; 

}



char get_sbcid ();

char  can_do_auto485 ( int sbc_id);

void print_help();

void init485_5300 ( unsigned long baud, int dl);

void init485_5400 (unsigned long , int);

void init485_5500 (unsigned long, int);

void process_args (int argc, char *argv[]);

/* auto485.c

 * parameters: -h(help), -d(debug), baud rate, DL (datalength)

 * returns: +1 on illegal argument(s)

 * returns -1: on error

 * returns: 0 on success, no errors

 */



unsigned char debug = 0;

int main (int argc, char *argv[]) {

	int c =0;

	char SBC = 0;

	unsigned long baud = 0;

	int dl = 0;



	while ( --argc > 0 && (*++argv)[0] == '-')

		while ( c = *++argv[0] )

			switch (c) {

			case 'h':

				print_help();

				return  0;

				break;

			case 'd':

				debug = TRUE;

				printf("debug set\n");

				break;

			default:

				printf("auto485.exe: illegal option.. see auto485.exe -h for help\n");

				return 1;

				break;

			}



	if (argc != 2  ) {

		printf("Usage: auto485.exe [-h, -d] BaudRate DL\n");

		printf("type auto485.exe -h for help\n");

		return 1;

	}



	if ( strstr("?", *argv) != NULL) {

		print_help();

		return 1;

	}

	sscanf( *argv++,"%ld", &baud);



	if ( strstr("?", *argv) != NULL) {

		print_help();

		return 1;

	}

	sscanf( *argv, "%d", &dl);

	

	if ( baud > 115200) {

		printf("sorry..Baud rate must not exceed 115200\n");

		return -1;

	}



	if ( !((dl == 8) || (dl == 9)) ){

		printf("sorry...DataLength can only be 8 or 9\n");

		return -1;

	}





/* can this product support auto485 ? */

 	SBC = get_sbcid();

	if (can_do_auto485( SBC ) == FALSE)  {

		printf("\a\nerror: This board does not support the auto485 feature\n");

		return -1;

	}



	switch (SBC) {

		case TS5300:

			init485_5300(baud, dl);

			break;

		case TS5400:

			init485_5400(baud, dl);

			break;

		case TS5500:

			init485_5500(baud, dl);

			break;

		default:

			printf("\aERROR: board isn't valid type\n");

			return -1;

	}



	return 0;

}



void init485_5300 ( unsigned long baud, int dl) {

	int value, io;

	float calculation;

	struct sigbit_struct {

		unsigned char lsb;

		unsigned char msb;

	};

		

	union uTV {

		struct sigbit_struct sigbit;

		unsigned int TV;/*big assumption here...

				int is a 2 byte (16 bit) int TurboC...

				in linux, this is a 32 bit value*/

	} TimerValue;



	calculation = (115200.0 / baud)* 2.0;

	TimerValue.TV = my_rint (calculation);



	if (debug) {

		printf("size of uTV: %d\n", sizeof(TimerValue));

		printf("TimerValue is %d, 0x%04x\n", 

				TimerValue.TV, TimerValue.TV);

		printf("TimerValue.msb is %d, 0x%02x\n", 

				TimerValue.sigbit.msb, TimerValue.sigbit.msb);

		printf("TimerValue.lsb is %d, 0x%02x\n", 

				TimerValue.sigbit.lsb, TimerValue.sigbit.lsb);

	}

	io = inportb(0x61);

	if (debug)

		printf("I/O 0x61 origionally is: 0x%02x\n", io);

	io |= 0x03;

	if (debug)

		printf("outport to io 0x61 value 0x%02x\n", io);

	outportb(0x61, io);



	if (debug)

		printf("outport to io 0x43 value 0xB6\n");

	outportb(0x43, 0xB6);



	if (debug)

		printf("outport 0x42 0x%02x\n", TimerValue.sigbit.lsb);

	outport(0x42, TimerValue.sigbit.lsb);



	if (debug)

		printf("outport to 0x42 0x%02x\n", TimerValue.sigbit.msb);

	outport(0x42, TimerValue.sigbit.msb);



	if (dl == 9)

		value = 0xC0;

	if (dl == 8)

		value = 0x80;

	outportb(0x75, value);

	if (debug)

		printf("wrote to 0x75 value of 0x%02x\n", value);



	printf("Successfully Initialized Automatic RS-485 mode on COM1\n");

}



void init485_5500 ( unsigned long baud, int dl) {

	int value;



	if (dl == 9)

		value = 0xC0;

	if (dl == 8)

		value = 0x80;

	outportb(0x75, value);

	if (debug)

		printf("wrote to 0x75 value of 0x%02x\n", value);



	printf("Successfully Initialized Automatic RS-485 mode on COM3\n");



}



void init485_5400 ( unsigned long baud, int dl) {

	float calculation;

	int value,io;

	

	struct sigbit_struct {

		unsigned char lsb;

		unsigned char msb;

	};

		

	union uTV {

		struct sigbit_struct sigbit;

		unsigned int TV;/*big assumption here...

				int is a 2 byte (16 bit) int TurboC...

				in linux, this is a 32 bit value*/

	} TimerValue;

	

	calculation = (115200.0 / baud * 3200.0) - 3.0 ;

	TimerValue.TV = my_rint (calculation);



	if (debug)

		printf("TimerValue is %d\n", TimerValue.TV);



	io = inportb(0x61);

	io |= 0x02; /* set bit 1 */

	io &= 0xFE; /* clear bit 0 */

	if (debug)

		printf("outb to 0x61, value 0x%02x\n", io);

	outportb(0x61, io);



	outportb(0x43, 0xB6);



	if (debug)

		printf("wrote to 0x42 0x%02x\n", TimerValue.sigbit.lsb);

	outport(0x42, TimerValue.sigbit.lsb);



	if (debug)

		printf("wrote to 0x42 0x%02x\n", TimerValue.sigbit.msb);

	outport(0x42, TimerValue.sigbit.msb);



	if (dl == 9)

		value = 0xC0;

	if (dl == 8)

		value = 0x80;

	outportb(0x75, value);

	if (debug)

		printf("wrote to 0x75 value of 0x%02x\n", value);







	printf("Successfully Initialized Automatic RS-485 mode on COM1\n");



}









char get_sbcid () {

	char id;

	id = inportb(0x74);

	if (debug)

		printf("board id: 0x%02x\n", id);

	return 	id;

}



char can_do_auto485 ( int sbc_id) {

	int i = 0;



	switch (sbc_id) {

		case TS5300:

			i = inportb(0x75);

			i &=0x08 ; // TS5300 must check for this as well

			if ( i != 0x08 )

				return FALSE;

		case TS5400:

		case TS5500:

			i = inportb(0x75);

			i &=0x02;

			if (i != 0x02) // is 485 enabled?

				return FALSE;

			break;

	}



	return TRUE;

}







void print_help() {

		printf("auto485.exe\n");

		printf("initializes the Automatic RS-485 feature of a\n");

		printf("TS-5300(Rev C or above), TS-5400, TS-5500\n");

		printf("usage:\n");

		printf("auto485.exe [-h, -d] BaudRate DL\n");

		printf("\t -h   help prints this help message and exits\n");

		printf("\t -d   debug Turn on Debug messages\n");

		printf("\t BaudRate: Desired BaudRate, e.g. 9600 or 115200\n");

		printf("\t DL: DataLength. Data length must include any parity");

		printf("\n\t\t bits, so a baud rate of \n");

		printf("\t\t 9600, E, 8, 1 would be 9 (8 bits + parity)\n\n");

}



